#pragma once
#include <string>
#include <gl\glew.h>
#include <glfw\glfw3.h>
#include <iostream>
#include <fstream>
#include <vector>
#include <memory>

#include <glm\glm.hpp>
#include <glm\gtc\type_ptr.hpp>
using namespace std;

enum ShaderType { VERT, FRAG, GEOM };

struct ShaderStruct
{
	ShaderType _type;
	string _shader;
	ShaderStruct(ShaderType type, const std::string& shader) :
		_type(type), _shader(shader) {};
};

class Shader
{
public:
	Shader(const std::string& vertexShader,
		const std::string& fragmentShader);
	Shader(std::vector<ShaderStruct>& shaders);
	~Shader();

	void useShader() const;

	void setUniform(std::string sName, float* fValues, int iCount = 1);
	void setUniform(std::string sName, const float fValue);
	void setUniform(std::string sName, int* iValues, int iCount = 1);
	void setUniform(std::string sName, const int iValue);
	void setUniform(std::string sName, const glm::vec3 vVector);
	void setUniform(std::string sName, const glm::mat4 mMatrix);
	GLuint programHandle;

private:
	GLuint _vertexHandle;
	GLuint _fragmentHandle;
	GLuint _geometryHandle;

	void loadShader(const std::string& shader,
		GLenum shaderType, GLuint& handle);
	void link();

};

