#version 430

layout(binding=0) uniform sampler2D uInputTex;
layout(binding=1) uniform sampler2D uSceneTex;

uniform float uDistortion;
uniform float uHaloWidth;
uniform float uGhostDispersal;

in vec2 textureCoords;

out vec4 oFragColor;

vec4 distortedTex(in sampler2D tex,  vec2 texCoord, in vec2 direction, in vec3 distortion) {
	return vec4(
		texture(tex, texCoord + direction * distortion.r).r,
		texture(tex, texCoord + direction * distortion.g).g,
		texture(tex, texCoord + direction * distortion.b).b,
		1.0
	);
}

void main() {
	
	// mimic the pivot of the lens flares around the image centre by flippling tex coords
	vec2 flippedTex = -textureCoords + vec2(1.0);
	
	// create the alignment vector to the image centre for the ghosts
	vec2 ghostAlign = (vec2(0.5) - flippedTex) * uGhostDispersal;

	// texel size and distortion factor + direction
	vec2 texelSize = 1.0 / vec2(textureSize(uInputTex, 0));
	vec3 distortionFac = vec3(-texelSize.x * uDistortion, 0.0, texelSize.x * uDistortion);
	vec2 distortionDir = normalize(ghostAlign);

	vec4 result = vec4(0.0);

	// create Ghosts
	for (int i = 0; i < 5; i++) {
		vec2 offs = fract(flippedTex + ghostAlign * float(i));
		
		float weight = length(vec2(0.5) - offs) / length(vec2(0.5));
		weight = pow(1.0f - weight, 10.0);

		result += distortedTex(uInputTex, offs, distortionDir, distortionFac) * weight;
	}

	/* disable halos as the tutors didn't like them
	vec2 vHalo = normalize(ghostAlign) * uHaloWidth;
	float weight = length(vec2(0.5) - fract(flippedTex + vHalo)) / length(vec2(0.5));
	weight = pow(1.0 - weight, 5.0);

	result += distortedTex(uInputTex, flippedTex + vHalo, distortionDir, distortionFac) * weight;
	*/

	oFragColor = texture(uSceneTex, textureCoords) + result;
}