#version 430 core

const float waveStrength =0.02f;
const float shineDamper = 20.0f;
const float reflectivity = 0.2f;
const vec3 lightColor = vec3(1.0f, 1.0f, 1.0f);

in vec4 clipSpace;
in vec2 textureCoords;
in vec3 fromLightVector;
in vec4 vertexColor;
in vec3 toCameraVector;
in vec3 vertPos;
in vec3 camPos;

layout(binding=0) uniform sampler2D reflectionTexture;
layout(binding=1) uniform sampler2D refractionTexture;
layout(binding=2) uniform sampler2D dudvMap;
layout(binding=3) uniform sampler2D normalMap;
layout(binding=4) uniform sampler2D depthMap;

uniform float moveFactor;
uniform int fog;
uniform vec4 skyColour;

out vec4 out_Color;

void main()
{
	//if(length(camPos-vertPos)>7.0f || length(camPos-vertPos)<-7.0f ){discard;}
	
	// transform coordinates
	vec2 ndc=(clipSpace.xy/clipSpace.w)/2.0f+0.5f;
	vec2 refractCoords=vec2(ndc.x, ndc.y);
	vec2 reflectCoords=vec2(ndc.x, -ndc.y);

	// water depth
	const float near = 0.1f;
	const float far = 40.0f; 
	float depth = texture(depthMap,refractCoords).r;
	float floorDistance = 2.0f * near * far / (far + near - (2.0f * depth - 1.0f) * (far - near));
	depth = gl_FragCoord.z;
	float waterDistance =  2.0f * near * far / (far + near - (2.0f * depth - 1.0f) * (far - near));
	float waterDepth = floorDistance - waterDistance;

	// waves
	vec2 distortedTexCoords = texture(dudvMap, vec2(textureCoords.x + moveFactor, textureCoords.y)).rg*0.1f;
	distortedTexCoords = textureCoords + vec2(distortedTexCoords.x, distortedTexCoords.y+moveFactor);
	vec2 totalDistortion = (texture(dudvMap, distortedTexCoords).rg * 2.0f - 1.0f) * waveStrength * clamp(waterDepth/5,0.0f,1.0f);

	// corrections
	refractCoords += totalDistortion;
	refractCoords = clamp(refractCoords,0.001f,0.999f);
	reflectCoords += totalDistortion;
	reflectCoords.x = clamp(reflectCoords.x,0.001f,0.999f);
	reflectCoords.y = clamp(reflectCoords.y,-0.999f,-0.001f);
	
	// fesnel effect
	vec4 reflectColor = texture(reflectionTexture,reflectCoords);
	vec4 refractColor = texture(refractionTexture,refractCoords);
	vec3 viewVector = normalize(toCameraVector);
	float refractiveFactor =dot(viewVector,vec3(0.0f,1.0f,0.0f));

	// murky water
	refractColor = mix(refractColor,vec4(0.22f,0.2f,0.13f,1.0f),clamp(waterDepth/5, 0.0, 1.0));

	// manipulate refractive factor
	refractiveFactor = pow(refractiveFactor,0.5f);

	// normal mapping
	vec4 normalMapColor = texture(normalMap,distortedTexCoords);
	vec3 normal = vec3(normalMapColor.r*2.0f-1.0f,normalMapColor.b,normalMapColor.g*2.0f-1.0f);
	normal = normalize(normal);
	out_Color = mix(reflectColor,refractColor,refractiveFactor);

	// specular highlights
	vec3 reflectedLight = reflect(normalize(fromLightVector), normal);
	float specular = max(dot(reflectedLight, viewVector), 0.0f);
	specular = pow(specular, shineDamper);
	vec3 specularHighlights = skyColour.xyz * specular * reflectivity;
	
	// blue tint
	out_Color = mix(out_Color,vec4(0.0f,0.23f,0.2f,1.0f),0.2f); 

	// fog interpolation if enabled
	if(fog==1){
	out_Color = vec4(mix(out_Color.rgb,skyColour.rgb,vertexColor.r), 0.8f);
	}
	
	// add specular highlights to color
	out_Color += vec4(specularHighlights,0.0f);
}